@@ -3,7 +3,7 @@
 # Name     : ModProxyPerlHtml.pm
 # Language : perl 5
 # Authors  : Gilles Darold, gilles at darold dot net
-# Copyright: Copyright (c) 2005-2012: Gilles Darold - All rights reserved -
+# Copyright: Copyright (c) 2005-2013: Gilles Darold - All rights reserved -
 # Description : This mod_perl2 module is a replacement for mod_proxy_html.c
 #		with far better URL HTML rewriting.
 # Usage    : See documentation in this file with perldoc.
@@ -29,7 +29,7 @@ use Apache2::ServerRec;
 use Apache2::URI;
 
 
-$Apache2::ModProxyPerlHtml::VERSION = '3.5';
+$Apache2::ModProxyPerlHtml::VERSION = '3.6';
 
 
 %Apache2::ModProxyPerlHtml::linkElements = (
@@ -84,6 +84,7 @@ sub handler
 		$contenttype ||= '(text\/javascript|text\/html|text\/css|text\/xml|application\/.*javascript|application\/.*xml)';
 		my $badcontenttype = $f->r->dir_config->get('ProxyHTMLExcludeContentType');
 		$badcontenttype ||= '(application\/vnd\.openxml)';
+		my @exclude = $f->r->dir_config->get('ProxyHTMLExcludeUri');
 
 		my $ct = $f->ctx;
 		$ct->{data} = '';
@@ -95,6 +96,9 @@ sub handler
 		}
 		$ct->{contenttype} = $contenttype;
 		$ct->{badcontenttype} = $badcontenttype;
+		foreach my $u (@exclude) {
+			push(@{$ct->{excluded}}, $u);
+		}
 		$f->ctx($ct);
 	}
 	# Thing we do on all invocations
@@ -106,86 +110,97 @@ sub handler
 	}
 	# Thing we do at end
 	if ($f->seen_eos) { 
-		# Skip content that should not have links
 		my $parsed_uri = $f->r->construct_url();
 		my $a_encoding = $f->r->headers_in->{'Accept-Encoding'} || '';
 		my $c_encoding = $f->r->headers_out->{'Content-Encoding'} || '';
 		my $ct = $f->r->headers_out->{'Content-type'} || '';
-	       	# if Accept-Encoding: gzip,deflate try to uncompress
-		if ( ($c_encoding =~ /gzip|deflate/) && ($ct =~ /$ctx->{contenttype}/is) && ($ct !~ /$ctx->{badcontenttype}/is) ) {
-			if ($debug) {
-				Apache2::ServerRec::warn("[ModProxyPerlHtml] Uncompressing $ct, Content-Encoding: $c_encoding\n");
-			}
-			use IO::Uncompress::AnyInflate qw(anyinflate $AnyInflateError) ;
-			my $output = '';
-			anyinflate  \$ctx->{data} => \$output or print STDERR "anyinflate failed: $AnyInflateError\n";
-			if ($ctx->{data} ne $output) {
-				$ctx->{data} = $output;
+
+		# Only proceed URLs that are not excluded from rewritter
+		if ( ($#{$ctx->{excluded}} == -1) || !grep($parsed_uri =~ /$_/i, @{$ctx->{excluded}}) ) {
+
+			# if Accept-Encoding: gzip,deflate try to uncompress
+			if ( ($c_encoding =~ /gzip|deflate/) && ($ct =~ /$ctx->{contenttype}/is) && ($ct !~ /$ctx->{badcontenttype}/is) ) {
+				if ($debug) {
+					Apache2::ServerRec::warn("[ModProxyPerlHtml] Uncompressing $ct, Content-Encoding: $c_encoding");
+				}
+				use IO::Uncompress::AnyInflate qw(anyinflate $AnyInflateError) ;
+				my $output = '';
+				anyinflate  \$ctx->{data} => \$output or print STDERR "anyinflate failed: $AnyInflateError\n";
+				if ($ctx->{data} ne $output) {
+					$ctx->{data} = $output;
+				} else {
+					$c_encoding = '';
+				}
 			} else {
 				$c_encoding = '';
 			}
-		} else {
-			$c_encoding = '';
-		}
-		my $refresh = $f->r->headers_out->{'Refresh'};
-		if ($refresh) {
-			foreach my $p (@{$ctx->{pattern}}) {
-				my ($match, $substitute) = split(/[\s\t]+/, $p, 2);
-				if ($refresh =~ s#([^\/:])$match#$1$substitute#) {
-					if ($debug) {
-						Apache2::ServerRec::warn("[ModProxyPerlHtml] Refresh header match '$match', substituted by: /$substitute/\n");
-					}
-				}
-			}
-			$f->r->headers_out->set('Refresh' => $refresh);
-		}
-		my $referer = $f->r->headers_out->{'Referer'};
-		if ($referer) {
-			foreach my $p (@{$ctx->{pattern}}) {
-				my ($match, $substitute) = split(/[\s\t]+/, $p, 2);
-				if ($referer =~ s#([^\/:])$match#$1$substitute#) {
-					if ($debug) {
-						Apache2::ServerRec::warn("[ModProxyPerlHtml] Referer header match '$match', substituted by: /$substitute/\n");
-					}
-				}
-			}
-			$f->r->headers_out->set('Referer' => $referer);
-		}
-		
-		if ( ($content_type =~ /$ctx->{contenttype}/is) && ($content_type !~ /$ctx->{badcontenttype}/is) ) {
-			if ($debug) {
-				Apache2::ServerRec::warn("[ModProxyPerlHtml] Content-type '$content_type' match: /$ctx->{contenttype}/is\n");
-			}
-			# Replace links if pattern match
-			foreach my $p (@{$ctx->{pattern}}) {
-				my ($match, $substitute) = split(/[\s\t]+/, $p, 2);
-				&link_replacement(\$ctx->{data}, $match, $substitute, $parsed_uri);
 
+			# Rewrite refresh command in header
+			my $refresh = $f->r->headers_out->{'Refresh'};
+			if ($refresh) {
+				foreach my $p (@{$ctx->{pattern}}) {
+					my ($match, $substitute) = split(/[\s\t]+/, $p, 2);
+					if ($refresh =~ s#([^\/:])$match#$1$substitute#) {
+						if ($debug) {
+							Apache2::ServerRec::warn("[ModProxyPerlHtml] Refresh header match '$match', substituted by: /$substitute/");
+						}
+					}
+				}
+				$f->r->headers_out->set('Refresh' => $refresh);
 			}
-			# Rewrite code if rewrite pattern match
-			foreach my $p (@{$ctx->{rewrite}}) {
-				my ($match, $substitute) = split(/[\s\t]+/, $p, 2);
-				&rewrite_content(\$ctx->{data}, $match, $substitute, $parsed_uri);
+
+			# Rewrite referer in header
+			my $referer = $f->r->headers_out->{'Referer'};
+			if ($referer) {
+				foreach my $p (@{$ctx->{pattern}}) {
+					my ($match, $substitute) = split(/[\s\t]+/, $p, 2);
+					if ($referer =~ s#([^\/:])$match#$1$substitute#) {
+						if ($debug) {
+							Apache2::ServerRec::warn("[ModProxyPerlHtml] Referer header match '$match', substituted by: /$substitute/");
+						}
+					}
+				}
+				$f->r->headers_out->set('Referer' => $referer);
+			}
+			
+			# Only parse content that should have hyperlinks to rewrite
+			if ( ($content_type =~ /$ctx->{contenttype}/is) && ($content_type !~ /$ctx->{badcontenttype}/is) ) {
+				if ($debug) {
+					Apache2::ServerRec::warn("[ModProxyPerlHtml] Content-type '$content_type' match: /$ctx->{contenttype}/is");
+				}
+				# Replace links if pattern match
+				foreach my $p (@{$ctx->{pattern}}) {
+					my ($match, $substitute) = split(/[\s\t]+/, $p, 2);
+					&link_replacement(\$ctx->{data}, $match, $substitute, $parsed_uri);
+
+				}
+				# Rewrite code if rewrite pattern match
+				foreach my $p (@{$ctx->{rewrite}}) {
+					my ($match, $substitute) = split(/[\s\t]+/, $p, 2);
+					&rewrite_content(\$ctx->{data}, $match, $substitute, $parsed_uri);
+				}
+			}
+
+			# Compress again data if require
+			if (($a_encoding =~ /gzip|deflate/) && ($c_encoding =~ /gzip|deflate/)) {
+				if ($debug) {
+					Apache2::ServerRec::warn("[ModProxyPerlHtml] Compressing output as Content-Encoding: $c_encoding");
+				}
+				if ($c_encoding =~ /gzip/) {
+					use IO::Compress::Gzip qw(gzip $GzipError) ;
+					my $output = '';
+					my $status = gzip \$ctx->{data} => \$output or die "gzip failed: $GzipError\n";
+					$ctx->{data} = $output;
+				} elsif ($c_encoding =~ /deflate/) {
+					use IO::Compress::Deflate qw(deflate $DeflateError) ;
+					my $output = '';
+					my $status = deflate \$ctx->{data} => \$output or die "deflate failed: $DeflateError\n";
+					$ctx->{data} = $output;
+				}
 			}
 		}
 
-		# Compress again data if require
-		if (($a_encoding =~ /gzip|deflate/) && ($c_encoding =~ /gzip|deflate/)) {
-			if ($debug) {
-				Apache2::ServerRec::warn("[ModProxyPerlHtml] Compressing output as Content-Encoding: $c_encoding\n");
-			}
-			if ($c_encoding =~ /gzip/) {
-				use IO::Compress::Gzip qw(gzip $GzipError) ;
-				my $output = '';
-				my $status = gzip \$ctx->{data} => \$output or die "gzip failed: $GzipError\n";
-				$ctx->{data} = $output;
-			} elsif ($c_encoding =~ /deflate/) {
-				use IO::Compress::Deflate qw(deflate $DeflateError) ;
-				my $output = '';
-				my $status = deflate \$ctx->{data} => \$output or die "deflate failed: $DeflateError\n";
-				$ctx->{data} = $output;
-			}
-		}
+		# Apply any change
 		$f->ctx($ctx);
 
 		# Dump datas out
@@ -193,11 +208,14 @@ sub handler
 		my $c = $f->c;
 		if ($c->keepalive == Apache2::Const::CONN_KEEPALIVE && $ctx->{data} && $c->keepalives > $ctx->{keepalives}) {
 			if ($debug) {
-				Apache2::ServerRec::warn("[ModProxyPerlHtml] Cleaning context for keep alive request\n");
+				Apache2::ServerRec::warn("[ModProxyPerlHtml] Cleaning context for keep alive request");
 			}
 			$ctx->{data} = '';
 			$ctx->{pattern} = ();
 			$ctx->{rewrite} = ();
+			$ctx->{excluded} = ();
+			$ctx->{contenttype} = '';
+			$ctx->{badcontenttype} = '';
 			$ctx->{keepalives} = $c->keepalives;
 		}
 			
@@ -268,7 +286,8 @@ sub rewrite_content
 	$/ = '';
 
 	# Rewrite things in code (case sensitive)
-	$$data =~ s/$pattern/$replacement/g;
+	$replacement = '"' . $replacement . '"';
+	$$data =~ s/$pattern/$replacement/eeg;
 
 	$/ = $old_terminator;
 
@@ -344,6 +363,10 @@ as follow:
     PerlInputFilterHandler Apache2::ModProxyPerlHtml
     PerlOutputFilterHandler Apache2::ModProxyPerlHtml
     SetHandler perl-script
+    # Use line below iand comment line above if you experience error:
+    # "Attempt to serve directory". The reason is that with SetHandler
+    # DirectoryIndex is not working 
+    # AddHandler perl-script *
     PerlSetVar ProxyHTMLVerbose "On"
     LogLevel Info
 
@@ -404,14 +427,27 @@ files. The result is that there could suffer of replacement inside and the file
 to prevent this you have the ProxyHTMLExcludeContentType configuration directive to exclude certain
 content-type. Here is the default value:
  
-	ProxyHTMLExcludeContentType	(application\/vnd\.openxml)
+	PerlAddVar ProxyHTMLExcludeContentType	(application\/vnd\.openxml)
 
 If you have problem with other content-type, use this directive. For example, as follow:
 
-	ProxyHTMLExcludeContentType	(application\/vnd\.openxml|application\/vnd\..*text)
+	PerlAddVar ProxyHTMLExcludeContentType	(application\/vnd\.openxml|application\/vnd\..*text)
 
 this regex will prevent any MS Office XML or text document to be parsed.
 
+Some javascript libraries like JQuery are wrongly rewritten by ModProxyPerlHtml.
+The problem is that those javascript code include some code and regex that are
+detected as links and rewritten. The only way to fix that is to exclude those
+files from the URL rewritter by using the "ProxyHTMLExcludeUri" configuration
+directive. For example:
+
+	PerlAddVar ProxyHTMLExcludeUri	jquery.min.js$
+	PerlAddVar ProxyHTMLExcludeUri	^.*\/jquery-lib\/.*$
+
+Any downloaded URI that contains the given regex will be returned asis without
+rewritting. You can use this directive multiple time like above to match different
+cases.
+
 =head1 LIVE EXAMPLE
 
 Here is the reverse proxy configuration I use to give access to Internet users
@@ -427,6 +463,10 @@ to internal applications:
     PerlInputFilterHandler Apache2::ModProxyPerlHtml
     PerlOutputFilterHandler Apache2::ModProxyPerlHtml
     SetHandler perl-script
+    # Use line below iand comment line above if you experience error:
+    # "Attempt to serve directory". The reason is that with SetHandler
+    # DirectoryIndex is not working 
+    # AddHandler perl-script *
     PerlSetVar ProxyHTMLVerbose "On"
     LogLevel Info
 
@@ -485,7 +525,7 @@ requests.
 
 =head1 COPYRIGHT
 
-Copyright (c) 2005-2012 - Gilles Darold
+Copyright (c) 2005-2013 - Gilles Darold
 
 All rights reserved.  This program is free software; you may redistribute
 it and/or modify it under the same terms as Perl itself.
